/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 * 
 * Copyright (©) 2005,2006,2007 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 2002,2007 Mark Wedel & Crossfire Development Team
 * Copyright (©) 1992,2007 Frank Tore Johansen
 * 
 * Deliantra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

/* Started file Sept 1996 - initialization of gods in form of a
 * linked list -b.t.
 */

#include <global.h>
#include <living.h>
#include <spells.h>

/* init_gods() - this takes a look at all of the archetypes to find
 * the objects which correspond to the GODS (type GOD) */

void
init_gods (void)
{
  LOG (llevDebug, "Initialising gods...\n");

  for_all_archetypes (at)
    if (at->type == GOD)
      add_god_to_list (at);

  LOG (llevDebug, "done.\n");
}

/* add_god_to_list()- called only from init_gods */
void
add_god_to_list (archetype *god_arch)
{
  if (!god_arch)
    {
      LOG (llevError, "ERROR: Tried to add null god to list!\n");
      return;
    }

  godlink *god = new godlink;

  // name=NULL;        /* how to describe the god to the player */
  // arch=NULL;        /* pointer to the archetype of this god */
  // id=0;             /* id of the god */
  // pantheon=NULL;    /* the group to which the god belongs (not implemented) */
  // next=NULL;        /* next god in this linked list */

  god->arch = god_arch;
  god->name = god_arch->object::name;

  if (!first_god)
    god->id = 1;
  else
    {
      god->id = first_god->id + 1;
      god->next = first_god;
    }

  first_god = god;

#ifdef DEBUG_GODS
  LOG (llevDebug, "Adding god %s (%d) to list\n", &god->name, god->id);
#endif
}

/* baptize_altar() - (cosmetically) change the name to that of the
 * god in question, then set the title for later use. -b.t.
 */
int
baptize_altar (object *op)
{
  char buf[MAX_BUF];

  /* if the title field is pre-set, then that altar is
   * already dedicated. */
  if (!op->title)
    {
      godlink *god = get_rand_god ();

      if (!god || !god->name)
        {
          LOG (llevError, "baptise_altar(): bizarre nameless god!\n");
          return 0;
        }
      /* if the object name hasnt' been changed, we tack on the gods name */
      if (!strcmp (op->name, op->arch->object::name))
        {
          sprintf (buf, "%s of %s", &op->name, &god->name);
          op->name = buf;
        }
      op->title = god->name;
      return 1;
    }
  return 0;
}

godlink *
get_rand_god (void)
{
  godlink *god = first_god;
  int i;

  if (god)
    for (i = rndm (god->id) + 1; god; god = god->next)
      if (god->id == i)
        break;

  if (!god)
    LOG (llevError, "get_rand_god(): can't find a random god!\n");

  return god;
}

/* pntr_to_god_obj() - returns a pointer to the object
 * We need to be VERY carefull about using this, as we
 * are returning a pointer to the CLONE object. -b.t.
 */
object *
pntr_to_god_obj (godlink *godlnk)
{
  return godlnk && godlnk->arch
    ? godlnk->arch
    : 0;
}

void
free_all_god (void)
{
  godlink *god, *godnext;

  LOG (llevDebug, "Freeing god information\n");
  for (god = first_god; god; god = godnext)
    {
      godnext = god->next;
      delete god;
    }
}
