/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 * 
 * Copyright (©) 2005,2006,2007 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 1992,2007 Frank Tore Johansen
 * 
 * Deliantra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

#include <global.h>

/*
 * Allocates a new objectlink structure, initialises it, and returns
 * a pointer to it.
 */

objectlink *
get_objectlink (void)
{
  objectlink *ol = new objectlink;

  ol->ob = 0;
  ol->next = 0;
  return ol;
}

/*
 * Allocates a new oblinkpt structure, initialises it, and returns
 * a pointer to it.
 */

oblinkpt *
get_objectlinkpt (void)
{
  oblinkpt *obp = new oblinkpt;

  obp->link = 0;
  obp->next = 0;
  obp->value = 0;
  return obp;
}

/*
 * Recursively frees all objectlinks
 */

void
free_objectlink (objectlink * ol)
{
  if (ol->next)
    free_objectlink (ol->next);

  delete ol;
}

/*
 * Recursively frees all linked list of objectlink pointers
 */

void
free_objectlinkpt (oblinkpt *obp)
{
  if (obp->next)
    free_objectlinkpt (obp->next);

  if (obp->link)
    free_objectlink (obp->link);

  delete obp;
}
